#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"


// Define triangle vertices relative to centre of screen
int VertexData[][3] =
{
	{-200, -220, 100}, {-200,  -20, 100}, { -50, -120, 100}, // Triangle 1
	{ 200, -220, 100}, { 200,  -20, 100}, {  50, -120, 100}, // Triangle 2
	{-200,  220, 100}, {-200,   20, 100}, { -50,  120, 100}, // Triangle 3
	{ 200,  220, 100}, { 200,   20, 100}, {  50,  120, 100}, // Triangle 4
};

// Define vertex  colours
int VertexColors[][3] =
{
	{255,   0,   0}, {  0, 255,   0}, {  0,   0, 255},  // Triangle 1
	{255,   0,   0}, {  0, 255,   0}, {  0,   0, 255},  // Triangle 2
	{255,   0,   0}, {  0, 255,   0}, {  0,   0, 255},  // Triangle 3
	{255,   0,   0}, {  0, 255,   0}, {  0,   0, 255},  // Triangle 4
}; 

// The number of vertices and colours
int iVertexCount = sizeof(VertexData)/(3*sizeof(int));
int iColourCount = sizeof(VertexColors)/(3*sizeof(int));

bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}


int main()
{
	// assert that these are the same
	assert((iVertexCount == iColourCount) && "Number of colours and vertices must be the same");
	
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	

	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(1536);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA

	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);

	// Set up the DMA packet to clear the screen. We want to clear to blue.
	SPS2Manager.InitScreenClear(0x20, 0, 0x20);

	while(g_bLoop)
	{
		// Fire off the previous frame's buffer (or init data if it's the first time)
		VIFDynamicDMA.Fire();

		pad_update(PAD_0);

		// Clear the screen
		SPS2Manager.BeginScene();
		
		// Start DIRECT mode transfer
		VIFDynamicDMA.StartDirect();
		
		// Add the GIFTag
		VIFDynamicDMA.Add128(GS_GIFTAG_BATCH(	iVertexCount,								//NLOOP
	 											1,											//EOP
	 											1,											//PRE
	 											11,											//PRIM
	 											GIF_FLG_PACKED,								//FLG
	 											GS_BATCH_2(GIF_REG_RGBAQ, GIF_REG_XYZ2)));	//BATCH					
			 
 
		// Loop through all vertices
	  	for (int i=0;i<iVertexCount;i++)
			{
			// Add the colour
	    	VIFDynamicDMA.Add128(PACKED_RGBA(	VertexColors[i][0],
	    										VertexColors[i][1],
	    										VertexColors[i][2],
	    										0x80));
	    	    	
	    	// Add the position   	   	
	    	VIFDynamicDMA.Add128(PACKED_XYZ2(	(2048 + VertexData[i][0])<< 4, 
	    										(2048 + VertexData[i][1])<< 4, 
	    										VertexData[i][2], 
	    										0));
	    	  	
	     	}
		
		// End the direct mode
		VIFDynamicDMA.EndDirect();
	
		// Swap the buffers
		SPS2Manager.EndScene();
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
	}
	
	// Cleanup the Pads
	pad_cleanup(PAD_0);
	
	return 0;
}

